﻿using MySql.Data.MySqlClient;
using System;

namespace MontoyaTech.MySqlPlus.Example
{
    public class Program
    {
        [MySqlRow("cars")]
        public class Car
        {
            [MySqlColumn(Id = true, Name = "id")]
            public ulong Id = 0;

            [MySqlColumn("make")]
            public string Make = null;

            [MySqlColumn("model")]
            public string Model = null;

            [MySqlColumn("year")]
            public uint Year = 0;

            [MySqlColumn("dateCreated", typeof(DateTime2UnixConverter))]
            public DateTime DateCreated = DateTime.UtcNow;
        }

        public class DateTime2UnixConverter : MySqlColumnConverter
        {
            public object ConvertFrom(object input)
            {
                if (input is ulong unix)
                {
                    if (unix == 0)
                        return DateTime.MinValue;

                    return new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc).AddSeconds(unix).ToUniversalTime();
                }
                else
                {
                    throw new NotSupportedException("Unsupported input object to convert from unix.");
                }
            }

            public object ConvertTo(object input)
            {
                if (input is DateTime dateTime)
                {
                    if (dateTime == DateTime.MinValue)
                        return 0;

                    if (dateTime == DateTime.MaxValue)
                        return ulong.MaxValue;

                    return (ulong)(dateTime.ToUniversalTime().Subtract(new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc))).TotalSeconds;
                }
                else
                {
                    throw new NotSupportedException("Unsupported input object to convert to unix.");
                }
            }
        }

        public static void Main(string[] args)
        {
            var session = new MySqlSession("");

            session.DeleteAll<Car>();

            session.Insert(new Car() { Make = "Chevy", Model = "Camaro", Year = 2011 });

            session.Insert(new Car() { Make = "GMC", Model = "Sierra", Year = 2000 });

            var cars = session.GetAll<Car>();

            foreach (var car in cars)
                Console.WriteLine($"Make: {car.Make}, Model: {car.Model}, Year: {car.Year}, DateCreated: {car.DateCreated}");

            cars[0].Make = "test";

            session.Update(cars[0]);

            foreach (var car in cars)
                session.Delete(car);

            Console.WriteLine("Done.");
            Console.ReadLine();
        }
    }
}